----------------
-- Window for Challenge
----------------

local AceGUI = LibStub("AceGUI-3.0")

-- Lua APIs
local pairs, assert, type = pairs, assert, type
local tblsort,tblremove = table.sort, table.remove

-- WoW APIs
local PlaySound = PlaySound
local CreateFrame, UIParent = CreateFrame, UIParent
local C_TimerAfter = C_Timer.After

----------------
-- Main Frame --
----------------
--[[
    Events :

]]
do
	local Type = "GoldGoblinLootGUIContainer-SortedScrollFrame"
	local Version = 1

	local function OnAcquire(self)
        self:ScrollFrameOnAcquire()
        self:SetDescending()
        self:SetSortFunction()
        self.descending = nil
	end

	local function OnRelease(self)
        self:ScrollFrameOnRelease()
        self:SetRefreshInterval()
        self:SetSortFunction()
    end

    local function OnShow(this)
		local self = this.obj
        self.isHidden = false
		if self and self.interval then
			self:SetRefreshInterval(self.interval)
		end
	end

    local function OnHide(this)
		local self = this.obj
        self.isHidden = true
	end

    local function SorterFunction(a, b)
        if a.GetSortValue and b.GetSortValue then
            return (a:GetSortValue() or 0) > (b:GetSortValue() or 0)
        else
            return (a:GetUserData("sortValue") or 0) > (b:GetUserData("sortValue") or 0)
        end
    end

    local function SorterFunctionDescanding(a, b)
        if a.GetSortValue and b.GetSortValue then
            return (a:GetSortValue() or 0) < (b:GetSortValue() or 0)
        else
            return (a:GetUserData("sortValue") or 0) < (b:GetUserData("sortValue") or 0)
        end
    end

    local function SetSortFunction(self, sortFunction)
        self.sortFunction = sortFunction
    end

    local function SetRefreshInterval(self, interval)
        -- set new interval
        self.interval = interval
        if not self.isRunning then
            self._OnRefresh()
        end
    end

    local function SetDescending(self, descending)
        self.descending = descending
    end

    local function SortChildrenList(self)
        self:Fire("BeforeListSort")
        if self.sortFunction then
            tblsort(self.children, self.sortFunction)
        else
            tblsort(self.children, self.descending and SorterFunctionDescanding or SorterFunction)
        end

        -- update children
        for i = 1, #self.children do
            if self.children[i].UpdateListRank then
                self.children[i]:UpdateListRank(i, true)
            end
        end

        self:DoLayout()

        self:Fire("AfterListSort")
    end

    local function ReleaseSingleChildren(self, childrenWidget)
        local children = self.children
        for i = 1,#children do
            if children[i] == childrenWidget then
                AceGUI:Release(children[i])
                tblremove(children, i)
                return
            end
		end
    end

    local function RemoveSingleChildren(self, childrenWidget)
        local children = self.children
        for i = 1,#children do
            if children[i] == childrenWidget then
                -- some clearing
                childrenWidget.frame:ClearAllPoints()
                childrenWidget.frame:Hide()
	            childrenWidget.frame:SetParent(UIParent)
                tblremove(children, i)
                self:DoLayout()
                return
            end
		end
    end

    local function GetNumEntrys(self)
        return #self.children
    end

    local function Constructor()
        -- use 'Window' as base
        local scrollFrame = AceGUI:Create("ScrollFrame")

        local self = scrollFrame
        local frame = scrollFrame.frame
		self.type = Type

        -- Hide/Show
        frame.obj = self
        frame:SetScript("OnShow", OnShow)
        frame:SetScript("OnHide", OnHide)

        -- save and overwirte
        self.ScrollFrameOnRelease = self.OnRelease
		self.ScrollFrameOnAcquire = self.OnAcquire
		self.OnRelease = OnRelease
        self.OnAcquire = OnAcquire
        self.SetDescending = SetDescending

        -- own things
        self.SortChildrenList = SortChildrenList
        self.SetRefreshInterval = SetRefreshInterval
        self.ReleaseSingleChildren = ReleaseSingleChildren
        self.RemoveSingleChildren = RemoveSingleChildren
        self.GetNumEntrys = GetNumEntrys
        self.SetSortFunction = SetSortFunction

        -- refresh function
        self._OnRefresh = function()
            if not self.interval or self.isHidden then
                self.isRunning = nil
                return
            end
            local children = self.children
            for i = 1,#children do
                if children[i].BeforePerformeUpdate then
                    children[i]:BeforePerformeUpdate()
                end
            end
            self:SortChildrenList()
            self.isRunning = true
            C_TimerAfter(self.interval, self._OnRefresh)
        end

		AceGUI:RegisterAsContainer(self)
		return self
	end

	AceGUI:RegisterWidgetType(Type,Constructor,Version)
end
